@@ -136,7 +136,7 @@ class ServiceTestCase(unittest.TestCase):
 
         self.assertTrue(mock_stop.called)
 
-    @mock.patch.object(service.Service, 'teardown')
+    @mock.patch.object(service.Service, 'teardown_service')
     def test_do_start_error(self, mock_teardown):
         """
         If an exception is raised when calling ``do_start`` while the service
@@ -232,8 +232,9 @@ class ServiceTestCase(unittest.TestCase):
 
         mock_greenlet.stop.assert_called_once_with()
 
+    @mock.patch.object(service.Service, 'stop')
     @mock.patch.object(service.Service, 'start')
-    def test_join(self, mock_start):
+    def test_join(self, mock_start, mock_stop):
         """
         Ensure that `join` works as one would expect.
         """
@@ -244,6 +245,7 @@ class ServiceTestCase(unittest.TestCase):
 
         mock_start.assert_called_once_with()
         thread.get.assert_called_once_with()
+        mock_stop.assert_called_once_with()
 
     def test_spawn_error(self):
         """
@@ -346,6 +348,71 @@ class ServiceTestCase(unittest.TestCase):
 
         self.assertEqual(my_service.services, [mock_service])
 
+    def test_teardown(self):
+        """
+        Teardown must be called before the state of the service is torn down.
+        """
+        class MyService(service.Service):
+            def do_teardown(self):
+                self.test.assertTrue(self.started)
+                self.executed = True
+
+        my_service = MyService()
+        my_service.test = self
+
+        my_service.start()
+        my_service.stop()
+
+        self.assertTrue(my_service.executed)
+
+    def test_teardown_not_started(self):
+        """
+        Teardown must not be called if the service has not been started.
+        """
+        class MyService(service.Service):
+            executed = False
+
+            def do_teardown(self):
+                self.executed = True
+
+        my_service = MyService()
+        my_service.test = self
+
+        my_service.stop()
+
+        self.assertFalse(my_service.executed)
+
+    def test_teardown_exception(self):
+        """
+        If there is an exception in `do_teardown`, the service must still be
+        torn down.
+        """
+        self.executed = False
+
+        class MyService(service.Service):
+            def do_teardown(self):
+                raise RuntimeError
+
+        def check_error(exc_type, exc_value, exc_tb):
+            self.assertIsInstance(exc_value, RuntimeError)
+            self.executed = True
+
+        my_service = MyService()
+        mock_service = mock.Mock()
+
+        my_service.add_service(mock_service)
+
+        my_service.start()
+
+        my_service.on('error', check_error)
+
+        # stopping the service will not raise the exception but will dump it
+        # to the logger
+        my_service.stop()
+
+        mock_service.stop.assert_called_once_with()
+        self.assertTrue(self.executed)
+
 
 class ConfigurableServiceTestCase(unittest.TestCase):
     """
@@ -382,3 +449,16 @@ class ConfigurableServiceTestCase(unittest.TestCase):
         my_service = MyService({'foo': 'baz'})
 
         self.assertEqual(my_service.config['foo'], 'baz')
+
+    def test_biloba_config(self):
+        """
+        Supplying a :ref:`biloba.config.Config` must set that as the config
+        instance for the service.
+        """
+        from biloba import config
+
+        cfg = config.Config()
+
+        svc = service.ConfigurableService(cfg)
+
+        self.assertIs(svc.config, cfg)
